//
// C++ Implementation: alloyonbutton
//
// Description:
//
//
// Author: Simon Ask Ulsnes <simon@ulsnes.dk>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <qtooltip.h>
#include <qpainter.h>
#include <qpixmap.h>

#include "enums.h"
#include "alloyonbutton.h"
#include "alloyonhandler.h"
#include "pixmaps.h"

#include "embeddata.h"

AlloyonButton::AlloyonButton(AlloyonClient *parent, const char *name, const QString& tip, ButtonType type) : QButton(parent->widget(), name), m_client(parent), m_lastMouse(0), m_type(type), hover(false), isOnAllDesktops(false), isMaximized(false)
{
	QToolTip::add(this, tip);
	setCursor(arrowCursor);
	setBackgroundMode(NoBackground);
	setFixedSize((type == ButtonMenu) ? 16 : AlloyonHandler::buttonSize(), AlloyonHandler::buttonSize());
}


AlloyonButton::~AlloyonButton()
{
}

void AlloyonButton::setTipText(const QString &tip)
{
	QToolTip::remove(this);
	QToolTip::add(this, tip);
}

QSize AlloyonButton::sizeHint() const
{
	return QSize((m_type == ButtonMenu) ? 16 : AlloyonHandler::buttonSize(), AlloyonHandler::buttonSize());
}

void AlloyonButton::enterEvent(QEvent *e)
{
	hover = true;
	repaint(false);
	QButton::enterEvent(e);
}

void AlloyonButton::leaveEvent(QEvent *e)
{
	hover = false;
	repaint(false);
	QButton::enterEvent(e);
}

void AlloyonButton::mousePressEvent(QMouseEvent *e)
{
	m_lastMouse = e->button();
	QMouseEvent me(e->type(), e->pos(), e->globalPos(), LeftButton, e->state());
	QButton::mousePressEvent(&me);
}

void AlloyonButton::mouseReleaseEvent(QMouseEvent *e)
{
	m_lastMouse = e->button();
	QMouseEvent me(e->type(), e->pos(), e->globalPos(), LeftButton, e->state());
	QButton::mouseReleaseEvent(&me);
}

void AlloyonButton::drawButton(QPainter *painter)
{
	if (!AlloyonHandler::initialized())
		return;

	bool active = m_client->isActive();
	int buttonSize = AlloyonHandler::buttonSize();
	int titleSize = AlloyonHandler::titleSize();

	QPixmap pufferPixmap;
	if (m_type == ButtonMenu)
		pufferPixmap.resize(16, titleSize);
	else
		pufferPixmap.resize(buttonSize, buttonSize);
	QPainter pufferPainter(&pufferPixmap);

	if (m_type == ButtonMenu)
	{
		QPixmap menu_icon = m_client->icon().pixmap(QIconSet::Small, QIconSet::Normal);
		pufferPainter.drawTiledPixmap(0, 0, 16, titleSize, active ? *Pixmaps::active_button_common_normal : *Pixmaps::inactive_button_common_normal);
		pufferPainter.drawPixmap(0, 2, menu_icon);
	}
	else
	{
		if (m_type == ButtonClose)
		{
			if (hover && !isDown())
			{
				pufferPainter.drawPixmap(0, 0, active ? *Pixmaps::active_button_close_hover : *Pixmaps::inactive_button_close_hover);
			}
			else if (isDown())
			{
				pufferPainter.drawPixmap(0, 0, active ? *Pixmaps::active_button_close_sunken : *Pixmaps::inactive_button_close_sunken);
			}
			else
			{
				pufferPainter.drawPixmap(0, 0, active ? *Pixmaps::active_button_close_normal : *Pixmaps::inactive_button_close_normal);
			}
		}
		else
		{
			if (hover && !isDown())
			{
				pufferPainter.drawPixmap(0, 0, active ? *Pixmaps::active_button_common_hover : *Pixmaps::inactive_button_common_hover);
			}
			else if (isDown())
			{
				pufferPainter.drawPixmap(0, 0, active ? *Pixmaps::active_button_common_sunken : *Pixmaps::inactive_button_common_sunken);
			}
			else
			{
				pufferPainter.drawPixmap(0, 0, active ? *Pixmaps::active_button_common_normal : *Pixmaps::inactive_button_common_normal);
			}
		}

		// Paint the little icon in the buttons
		int center = (buttonSize-8)/2;
		if (isDown())
		{
			switch (m_type)
			{
				case ButtonHelp:
					pufferPainter.drawImage(center, center+1, uic_findImage("question_pressed.png"));
					break;
				case ButtonClose:
					pufferPainter.drawImage(center, center+1, uic_findImage("close_pressed.png"));
					break;
				case ButtonMax:
					pufferPainter.drawImage(center, center+1, uic_findImage(isMaximized ? "restore_pressed.png" : "up_pressed.png"));
					break;
				case ButtonMin:
					pufferPainter.drawImage(center, center+1, uic_findImage("down_pressed.png"));
					break;
				case ButtonOnAllDesktops:
					pufferPainter.drawImage(center, center+1, uic_findImage(isOnAllDesktops ? "unsticky_pressed.png" : "sticky_pressed.png"));
					break;
				default:
					break;
			}
		}
		else
		{
			switch (m_type)
			{
				case ButtonHelp:
					pufferPainter.drawImage(center, center, uic_findImage("question.png"));
					break;
				case ButtonClose:
					pufferPainter.drawImage(center, center, uic_findImage("close.png"));
					break;
				case ButtonMax:
					pufferPainter.drawImage(center, center, uic_findImage(isMaximized ? "restore.png" : "up.png"));
					break;
				case ButtonMin:
					pufferPainter.drawImage(center, center, uic_findImage("down.png"));
					break;
				case ButtonOnAllDesktops:
					pufferPainter.drawImage(center, center, uic_findImage(isOnAllDesktops ? "unsticky.png" : "sticky.png"));
					break;
				default:
					break;
			}
		}

	}

	pufferPainter.end();
	painter->drawPixmap(0, 0, pufferPixmap);
}
#include "alloyonbutton.moc"
