//
// C++ Implementation: alloyonclient
//
// Description:
//
//
// Author: Simon Ask Ulsnes <simon@ulsnes.dk>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <klocale.h>
#include <kpixmap.h>
#include <kpixmapeffect.h>

#include <qcursor.h>
#include <qfontmetrics.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qtooltip.h>

#include "alloyonclient.h"
#include "alloyonhandler.h"
#include "alloyonbutton.h"
#include "pixmaps.h"

AlloyonClient::AlloyonClient(KDecorationBridge *bridge, KDecorationFactory *factory)
 : KDecoration(bridge, factory), m_titleBar(0)
{
}

AlloyonClient::~AlloyonClient()
{
	// Delete buttons from memory.
	for (int n=0; n<ButtonTypeCount; n++)
	{
		if (m_button[n]) delete m_button[n];
	}
}

void AlloyonClient::init()
{
	createMainWidget(WResizeNoErase | WRepaintNoErase);
	widget()->installEventFilter(this);
	widget()->setBackgroundMode(NoBackground);	// to avoid flicker

	// Set up layout
	QGridLayout *mainLayout = new QGridLayout(widget(), 4, 3);	// 4 x 3 grid
	QHBoxLayout *titleLayout = new QHBoxLayout();
	QVBoxLayout *leftButtonLayout = new QVBoxLayout();
	QHBoxLayout *leftButtonLayout2 = new QHBoxLayout();
	leftButtonLayout->setSpacing(AlloyonHandler::buttonSpacing());
	QVBoxLayout *rightButtonLayout = new QVBoxLayout();
	QHBoxLayout *rightButtonLayout2 = new QHBoxLayout();
	rightButtonLayout->setSpacing(AlloyonHandler::buttonSpacing());

	mainLayout->setResizeMode(QLayout::FreeResize);

	m_titleBar = new QSpacerItem(1, AlloyonHandler::titleSize(), QSizePolicy::Expanding, QSizePolicy::Fixed);
	m_leftSpacer = new QSpacerItem(AlloyonHandler::borderSize(), 1, QSizePolicy::Expanding, QSizePolicy::Minimum);
	m_rightSpacer = new QSpacerItem(AlloyonHandler::borderSize(), 1, QSizePolicy::Expanding, QSizePolicy::Minimum);
	m_topSpacer = new QSpacerItem(1, 1); // 2nd argument is the size of the frame on the top
	m_bottomSpacer = new QSpacerItem(1, AlloyonHandler::borderSize());
	m_leftButtonSpacer = new QSpacerItem(AlloyonHandler::buttonSpacing(), AlloyonHandler::titleSize());
	m_rightButtonSpacer = new QSpacerItem(AlloyonHandler::buttonSpacing(), AlloyonHandler::titleSize());
	m_leftButtonSpacer2 = new QSpacerItem(1, (AlloyonHandler::titleSize()-AlloyonHandler::buttonSize())/2, QSizePolicy::Minimum, QSizePolicy::Fixed);
	m_rightButtonSpacer2 = new QSpacerItem(1, (AlloyonHandler::titleSize()-AlloyonHandler::buttonSize())/2, QSizePolicy::Minimum, QSizePolicy::Fixed);

	mainLayout->addItem(m_topSpacer, 0, 1);
	mainLayout->addItem(m_bottomSpacer, 3, 1);
	mainLayout->addItem(m_leftSpacer, 1, 0);
	mainLayout->addItem(m_rightSpacer, 1, 2);

	mainLayout->addLayout(titleLayout, 1, 1);
	if (isPreview())
	{
		mainLayout->addWidget(new QLabel(i18n("<center><b>Alloyon</b></center>"), widget()), 2, 1);
	}
	else
	{
		mainLayout->addItem(new QSpacerItem(0, 0), 2, 1);
	}

	// Make the window stretch
	mainLayout->setRowStretch(2, 10);
	mainLayout->setColStretch(1, 10);

	titleLayout->addItem(m_leftButtonSpacer); // Left button margin
	titleLayout->addLayout(leftButtonLayout); // Contains the left buttons
	titleLayout->addItem(m_titleBar); // the text
	titleLayout->addLayout(rightButtonLayout); // Contains the right buttons
	titleLayout->addItem(m_rightButtonSpacer); // Right button margin

	for (int n=0; n<ButtonTypeCount; n++) m_button[n] = 0; // reset buttons

	// Add the left buttons
	leftButtonLayout->addItem(m_leftButtonSpacer2);
	leftButtonLayout->addLayout(leftButtonLayout2);
	addButtons(leftButtonLayout2, options()->customButtonPositions() ? options()->titleButtonsLeft() : QString(default_left));

	// Add the right buttons
	rightButtonLayout->addItem(m_rightButtonSpacer2);
	rightButtonLayout->addLayout(rightButtonLayout2);
	addButtons(rightButtonLayout2, options()->customButtonPositions() ? options()->titleButtonsRight() : QString(default_right));
}

bool AlloyonClient::eventFilter(QObject *o, QEvent *e)
{
	if (o != widget())
		return false;

	switch (e->type())
	{
		case QEvent::Resize:
			resizeEvent(static_cast<QResizeEvent*>(e));
			return true;
		case QEvent::Paint:
			paintEvent(static_cast<QPaintEvent*>(e));
			return true;
		case QEvent::MouseButtonDblClick:
			mouseDoubleClickEvent(static_cast<QMouseEvent*>(e));
			return true;
		case QEvent::MouseButtonPress:
			processMousePressEvent(static_cast<QMouseEvent*>(e));
			return true;
		case QEvent::Show:
			showEvent(static_cast<QShowEvent*>(e));
			return true;

		default:
			return false;
	}
}

void AlloyonClient::addButtons(QBoxLayout *layout, const QString &s)
{
	if (s.length() > 0)
	{
		for (unsigned n=0; n<s.length(); n++)
		{
			switch (s[n])
			{
				case 'M': // Menu
					if (!m_button[ButtonMenu])
					{
						m_button[ButtonMenu] = new AlloyonButton(this, "menu", i18n("Menu"), ButtonMenu);
						connect(m_button[ButtonMenu], SIGNAL(pressed()), this, SLOT(menuButtonPressed()));
						layout->addWidget(m_button[ButtonMenu], 0, Qt::AlignHCenter | Qt::AlignTop);
					}
					break;
				case 'S': // OnAllDesktops
					if (!m_button[ButtonOnAllDesktops])
					{
						m_button[ButtonOnAllDesktops] = new AlloyonButton(this, "onAllDesktops", isOnAllDesktops() ? i18n("Not On All Desktops") : i18n("On All Desktops"), ButtonOnAllDesktops);
						m_button[ButtonOnAllDesktops]->setOnAllDesktops(isOnAllDesktops());
						connect(m_button[ButtonOnAllDesktops], SIGNAL(clicked()), this, SLOT(toggleOnAllDesktops()));
						layout->addWidget(m_button[ButtonOnAllDesktops], 0, Qt::AlignHCenter | Qt::AlignTop);
					}
					break;
				case 'H': // Help
					if (!m_button[ButtonHelp] && providesContextHelp())
					{
						m_button[ButtonHelp] = new AlloyonButton(this, "help", i18n("Help"), ButtonHelp);
						connect(m_button[ButtonHelp], SIGNAL(clicked()), this, SLOT(showContextHelp()));
						layout->addWidget(m_button[ButtonHelp], 0, Qt::AlignHCenter | Qt::AlignTop);
					}
					break;
				case 'I': // Minimize
					if ((!m_button[ButtonMin]) && isMinimizable())
                  			{
                      				m_button[ButtonMin] = new AlloyonButton(this, "minimize", i18n("Minimize"), ButtonMin);
                      				connect(m_button[ButtonMin], SIGNAL(clicked()), this, SLOT(minimize()));
                      				layout->addWidget(m_button[ButtonMin], 0, Qt::AlignHCenter | Qt::AlignTop);
                  			}
					break;
				case 'A': // Maxmize
					if ((!m_button[ButtonMax]) && isMaximizable())
                  			{
                      			m_button[ButtonMax] = new AlloyonButton(this, "maximize", (maximizeMode()!=MaximizeRestore)?i18n("Minimize"):i18n("Maximize"), ButtonMax);
                      			connect(m_button[ButtonMax], SIGNAL(clicked()), this, SLOT(maxButtonPressed()));
                      			layout->addWidget(m_button[ButtonMax], 0, Qt::AlignHCenter | Qt::AlignTop);
                  			}
                  			break;
				case 'X': // Close
                  			if ((!m_button[ButtonClose]) && isCloseable())
                  			{
                      				m_button[ButtonClose] = new AlloyonButton(this, "close", i18n("Close"), ButtonClose);
                      				connect(m_button[ButtonClose], SIGNAL(clicked()), this, SLOT(closeWindow()));
                      				layout->addWidget(m_button[ButtonClose], 0, Qt::AlignHCenter | Qt::AlignTop);
                  			}
                  			break;
				case '_': // Spacer
					layout->addSpacing(AlloyonHandler::buttonSpacing());
			}
		}
	}
}

void AlloyonClient::paintEvent(QPaintEvent*)
{
	if (!AlloyonHandler::initialized()) return;

	bool active = isActive();

	QPainter painter(widget());
	QColorGroup group;
	group = options()->colorGroup(ColorTitleBar, active);
	QColor color2 = group.background();
	//QColor color1 = Pixmaps::hsvRelative(color2, 0, 20, -50);

	/*QImage *title_image = new QImage();
	*title_image = active ? Pixmaps::active_titlebar->convertToImage() : Pixmaps::inactive_titlebar->convertToImage();
	QColor color1 = title_image->pixel(0, 0);
	QColor color2 = title_image->pixel(0, AlloyonHandler::titleSize()-1);
	delete title_image;*/

	QRect titleRect(m_titleBar->geometry());
	QRect topRect(m_topSpacer->geometry());
	QRect leftRect(m_leftSpacer->geometry());
	QRect rightRect(m_rightSpacer->geometry());
	QRect bottomRect(m_bottomSpacer->geometry());

	QFontMetrics fm(options()->font(active, false));
	int captionMetricWidth = fm.width(caption());	// Necessary width
	int captionWidth = captionMetricWidth + AlloyonHandler::titleShadowSize() + 2;	// available width (+ shadow)

	painter.drawTiledPixmap(0, titleRect.top(), topRect.width()+leftRect.width()+rightRect.width(), titleRect.bottom()-titleRect.top(), active ? *Pixmaps::active_titlebar : *Pixmaps::inactive_titlebar);

	if (titleRect.width() > 0)	// do we even have to paint the title?
	{
		QPixmap pufferPixmap;
		pufferPixmap.resize(captionWidth+AlloyonHandler::titleShadowSize(), titleRect.height()-1);

		int flags = (pufferPixmap.width() < captionMetricWidth) ? AlignLeft | AlignHCenter : AlignCenter;

		QPainter pufferPainter(&pufferPixmap);
		pufferPainter.drawTiledPixmap(pufferPixmap.rect(), active ? *Pixmaps::active_titlebar : *Pixmaps::inactive_titlebar);
		pufferPainter.setFont(options()->font(active, false));

		// move the text 1 px down and to the right
		pufferPainter.translate(2, 1);

		// draw title shadow
		if (AlloyonHandler::titleShadow())
		{
			pufferPainter.translate(AlloyonHandler::titleShadowSize(), AlloyonHandler::titleShadowSize());
			pufferPainter.setPen(group.background().dark(200));
			pufferPainter.drawText(pufferPixmap.rect(), flags, caption());
			pufferPainter.translate(-(AlloyonHandler::titleShadowSize()), -(AlloyonHandler::titleShadowSize()));
		}

		// draw title text
		pufferPainter.setPen(options()->color(ColorFont, active));
		pufferPainter.drawText(pufferPixmap.rect(), flags, caption());

		pufferPainter.end();
		painter.drawPixmap(titleRect.left(), titleRect.top(), pufferPixmap);
	}

	// Now we're going to draw the frame...

	// The light line on the very top, which is dark if we're maximized
	if(maximizeMode() != MaximizeFull)
	{
        	painter.setPen(Pixmaps::hsvRelative(color2, 0, -80, -10) );
        	painter.drawLine(0, 0, topRect.width()+leftRect.width()+rightRect.width(), 0);
    	}
    	else
    	{
		painter.setPen(Pixmaps::hsvRelative(color2, 0, 0, -50));
		painter.drawLine(0, 0, topRect.width()+leftRect.width()+rightRect.width(), 0);
	}

	// Left, top (light)
	if (leftRect.width() >= 1)
	{
		painter.drawTiledPixmap(leftRect.left(), titleRect.top(), 1, titleRect.bottom()-titleRect.top(), active ? *Pixmaps::active_titlebar_left : *Pixmaps::inactive_titlebar_left);
	}

	// Left, bottom, right (light)
	//painter.setPen(Pixmaps::hsvRelative(color2, 0, -50, +40));
	//painter.setPen(QColor(255, 0, 0));
	painter.setPen(Pixmaps::hsvRelative(color2, 0, -80, +40));
	if (leftRect.width() >= 1)
	{
		painter.drawLine(leftRect.left(), titleRect.bottom(), leftRect.left(), bottomRect.bottom()-1);
	}
	if (bottomRect.height() >= 2)
	{
		painter.drawLine(leftRect.right()+1, bottomRect.top(), rightRect.left()-1, bottomRect.top());
	}
	if (rightRect.width() >= 2)
	{
		painter.drawLine(rightRect.left(), titleRect.bottom()+1, rightRect.left(), bottomRect.top());
	}

	// Top, left, bottom, right (dark)
	//painter.setPen(Pixmaps::hsvRelative(color2, 0, 0, -20) );
	//painter.setPen(QColor(0, 255, 0));
	painter.setPen(Pixmaps::hsvRelative(color2, 0, -80, -50));
	painter.drawLine(leftRect.right()+1, titleRect.bottom(), rightRect.left()-1, titleRect.bottom());
	if(leftRect.width() >= 2)
	{
		painter.drawLine(leftRect.right(), titleRect.bottom(), leftRect.right(), bottomRect.top());
	}
	if(bottomRect.width() >= 4)
	{
		painter.drawLine(leftRect.left()+1, bottomRect.bottom(), rightRect.right(), bottomRect.bottom());
	}
	if(rightRect.width() >= 1)
	{
		painter.drawLine(rightRect.right(), titleRect.bottom(), rightRect.right(), bottomRect.bottom());
	}

	// Right, top (dark)
	if(rightRect.width() >= 1)
	{
		painter.drawTiledPixmap(rightRect.right(), titleRect.top(), 1, titleRect.bottom()-titleRect.top(), active ? *Pixmaps::active_titlebar_right : *Pixmaps::inactive_titlebar_right);
	}

	// Fill the rest with the base colour
	painter.setPen(color2);
	if(leftRect.width() >= 2)
	{
		painter.drawPoint(leftRect.right(), bottomRect.top());
	}
	if(rightRect.width() >= 2)
	{
	        painter.drawPoint(rightRect.left(), titleRect.bottom());
	}
	if(leftRect.width() >= 1)
	{
		painter.drawPoint(leftRect.left(), bottomRect.bottom());
	}
	if(rightRect.width() >= 1)
	{
		painter.drawPoint(rightRect.right(), topRect.top());
	}

	QRect frame;
	if(leftRect.width() >= 3)
	{
		frame.setCoords(leftRect.left()+1, titleRect.bottom(), leftRect.right()-1, bottomRect.bottom()-1);
		painter.fillRect(frame, color2);
	}
	if(rightRect.width() >= 3)
	{
		frame.setCoords(rightRect.left()+1, titleRect.bottom(), rightRect.right()-1, bottomRect.bottom()-1);
		painter.fillRect(frame, color2);
	}
	if(bottomRect.height() >= 3)
	{
		frame.setCoords(leftRect.right(), bottomRect.top()+1, rightRect.left(), bottomRect.bottom()-1);
		painter.fillRect(frame, color2);
	}

	// Draw resizers
	if (isResizable())
	{
		QPixmap *detailPtr = 0;
		// Choose the proper pixmap from the border size
		detailPtr = (bottomRect.height() < 5 || leftRect.width() < 5 || rightRect.width() < 5) ? (active ? Pixmaps::active_resizer_detail_4px : Pixmaps::inactive_resizer_detail_4px) : (active ? Pixmaps::active_resizer_detail_5px : Pixmaps::inactive_resizer_detail_5px);
		int detailSize = (bottomRect.height() < 5 || leftRect.width() < 5 || rightRect.width() < 5) ? 2 : 3;

		if (bottomRect.height() > 3)
		{
			painter.drawPixmap(leftRect.left()+1, bottomRect.bottom()-detailSize, *detailPtr);
			painter.drawPixmap(rightRect.right()-detailSize, bottomRect.bottom()-detailSize, *detailPtr);
			painter.drawPixmap(leftRect.left()+1+detailSize+2, bottomRect.bottom()-detailSize, *detailPtr);
			painter.drawPixmap(rightRect.right()-detailSize-2-detailSize, bottomRect.bottom()-detailSize, *detailPtr);
		}

		if (leftRect.width() > 3)
		{
			painter.drawPixmap(leftRect.left()+1, bottomRect.bottom()-detailSize-2-detailSize, *detailPtr);
		}
		else if (bottomRect.height() > 3)
		{
			painter.drawPixmap(leftRect.left()+1+detailSize+2+detailSize+2, bottomRect.bottom()-detailSize, *detailPtr);
		}

		if (rightRect.width() > 3)
		{
			painter.drawPixmap(rightRect.right()-detailSize, bottomRect.bottom()-detailSize-2-detailSize, *detailPtr);
		}
		else if (bottomRect.height() > 3)
		{
			painter.drawPixmap(rightRect.right()-detailSize-2-detailSize-2-detailSize, bottomRect.bottom()-detailSize, *detailPtr);
		}
	}
}

void AlloyonClient::showEvent(QShowEvent*)
{
	widget()->update();
}

void AlloyonClient::resizeEvent(QResizeEvent*)
{
	if ((widget()->isVisibleToTLW()) && (!widget()->testWFlags(WStaticContents)))
	{
		QRegion region = widget()->rect();
		region.subtract(m_titleBar->geometry());
		widget()->erase(region);
	}
}

void AlloyonClient::captionChange()
{
	widget()->repaint(m_titleBar->geometry(), false);
}

void AlloyonClient::mouseDoubleClickEvent(QMouseEvent *e)
{
	if (m_titleBar->geometry().contains(e->pos()))
	{
		titlebarDblClickOperation();
	}
}

AlloyonClient::Position AlloyonClient::mousePosition(const QPoint &point) const
{
	const int corner = 24;
	AlloyonClient::Position pos = PositionCenter;

	// Often needed coordinates...
	QRect titleRect(m_titleBar->geometry());
	QRect topRect(m_topSpacer->geometry());
	QRect leftRect(m_leftSpacer->geometry());
	QRect rightRect(m_rightSpacer->geometry());
	QRect bottomRect(m_bottomSpacer->geometry());

	if ((m_leftButtonSpacer2->geometry().height() > 0) ? (point.y() <= (m_leftButtonSpacer2->geometry().bottom())) : (point.y() <= topRect.bottom()))
	{
		// We're inside the top frame.
		if (point.x() <= corner)
			pos = PositionTopLeft;
		else if (point.x() >= (width()-corner))
			pos = PositionTopRight;
		else
			pos = PositionTop;
	}
	else if (topRect.height() <= 0 && point.y() <= 2)
	{
		// We're inside the top frame, probably maximized.
		if ((point.x() >= titleRect.left()) && (point.x() <= titleRect.left()+corner))
			pos = PositionLeft;
		else if ((point.x() >= titleRect.right()-corner) && (point.x() <= titleRect.right()))
			pos = PositionTopRight;
		else if ((point.x() > titleRect.left()) && (point.x() < titleRect.right()))
			pos = PositionTop;
	}
	else if (point.y() >= bottomRect.top())
	{
		// Inside handle
		if (point.x() <= corner)
			pos = PositionBottomLeft;
		else if (point.x() >= (width()-corner))
			pos = PositionBottomRight;
		else
			pos = PositionBottom;
	}
	else if (point.x() <= leftRect.right())
	{
		// Inside the left frame.
		if (point.y() <= corner)
			pos = PositionTopLeft;
		else if (point.y() >= (height()-corner))
			pos = PositionBottomLeft;
		else
			pos = PositionLeft;
	}
	else if (point.x() >= rightRect.left())
	{
		// Inside the right frame.
		if (point.y() <= corner)
			pos = PositionTopRight;
		else if (point.y() >= (height()-corner))
			pos = PositionBottomRight;
		else
			pos = PositionRight;
	}
	else
	{
		// Inside the frame somewhere...
		pos = PositionCenter;
	}

	return pos;
}

void AlloyonClient::iconChange()
{
	if (m_button[ButtonMenu])
	{
		m_button[ButtonMenu]->repaint(false);
	}
}

void AlloyonClient::activeChange()
{
	// Repaint the buttons when state changes
	for (int n=0; n<ButtonTypeCount; n++)
		if (m_button[n]) m_button[n]->repaint(false);
	widget()->repaint(false);
}

void AlloyonClient::maximizeChange()
{
	const bool m = (maximizeMode() != MaximizeRestore);
	if (m_button[ButtonMax])
	{
		m_button[ButtonMax]->setMaximized(m);
		m_button[ButtonMax]->setTipText(m ? i18n("Restore") : i18n("Maximize"));
	}
}

void AlloyonClient::desktopChange()
{
	if (m_button[ButtonOnAllDesktops])
	{
		m_button[ButtonOnAllDesktops]->setOnAllDesktops(isOnAllDesktops());
		m_button[ButtonOnAllDesktops]->setTipText(isOnAllDesktops() ? i18n("Not On All Desktops") : i18n("On All Desktops"));
	}
}

void AlloyonClient::maxButtonPressed()
{
	if (m_button[ButtonMax])
	{
		switch (m_button[ButtonMax]->lastMousePress())
		{
			case MidButton:
				maximize(maximizeMode() ^ MaximizeVertical);
				break;
			case RightButton:
				maximize(maximizeMode() ^ MaximizeHorizontal);
				break;
			default:
				maximize(maximizeMode() == MaximizeFull ? MaximizeRestore : MaximizeFull);
		}
	}
}

void AlloyonClient::menuButtonPressed()
{
	if (m_button[ButtonMenu])
	{
		QPoint pt(m_button[ButtonMenu]->rect().bottomLeft().x(), m_button[ButtonMenu]->rect().bottomLeft().y());
		showWindowMenu(m_button[ButtonMenu]->mapToGlobal(pt));
		m_button[ButtonMenu]->setDown(false);
	}
}

QSize AlloyonClient::minimumSize() const
{
	return widget()->minimumSize();
}

void AlloyonClient::borders(int& left, int& right, int& top, int& bottom) const
{
	if ((maximizeMode() == MaximizeFull) && !options()->moveResizeMaximizedWindows())
	{
		m_leftSpacer->changeSize( 0, 1, QSizePolicy::Expanding, QSizePolicy::Minimum );
		m_rightSpacer->changeSize( 0, 1, QSizePolicy::Expanding, QSizePolicy::Minimum );
		m_leftButtonSpacer->changeSize( 0, 1);
		m_rightButtonSpacer->changeSize( 0, 1);
		m_topSpacer->changeSize(1, 0);
		m_bottomSpacer->changeSize(1, 3);
		m_titleBar->changeSize(1, AlloyonHandler::titleSize()-(AlloyonHandler::titleSize()-AlloyonHandler::buttonSize())/2, QSizePolicy::Expanding, QSizePolicy::Fixed);
		m_leftButtonSpacer2->changeSize( 1, 0, QSizePolicy::Minimum, QSizePolicy::Fixed);
		m_rightButtonSpacer2->changeSize( 1, 0, QSizePolicy::Minimum, QSizePolicy::Fixed);

		left = right = 0;
		bottom = 3;
		top = AlloyonHandler::titleSize()-(AlloyonHandler::titleSize()-AlloyonHandler::buttonSize())/2;
	}
	else if ((maximizeMode() == MaximizeHorizontal) && !options()->moveResizeMaximizedWindows())
	{
		// We've been maximized - shrink the sideways spacers
		m_leftSpacer->changeSize( 1, 1, QSizePolicy::Expanding, QSizePolicy::Minimum );
		m_rightSpacer->changeSize( 1, 1, QSizePolicy::Expanding, QSizePolicy::Minimum );
		m_topSpacer->changeSize(1, 1);
		m_bottomSpacer->changeSize(1, AlloyonHandler::borderSize() );
		m_leftButtonSpacer->changeSize( 0, 1);
		m_rightButtonSpacer->changeSize( 0, 1);
		m_titleBar->changeSize(1, AlloyonHandler::titleSize(), QSizePolicy::Expanding, QSizePolicy::Fixed);
		m_leftButtonSpacer2->changeSize(1, (AlloyonHandler::titleSize()-AlloyonHandler::buttonSize())/2, QSizePolicy::Minimum, QSizePolicy::Fixed);
		m_rightButtonSpacer2->changeSize(1, (AlloyonHandler::titleSize()-AlloyonHandler::buttonSize())/2, QSizePolicy::Minimum, QSizePolicy::Fixed);

		left = right = 1;
		bottom = AlloyonHandler::borderSize();
		top = 1 + AlloyonHandler::titleSize();
	}
	else if ((maximizeMode() == MaximizeVertical) && !options()->moveResizeMaximizedWindows())
	{
		// We've been maximized - shrink the sideways spacers
		m_leftSpacer->changeSize(AlloyonHandler::borderSize(), 1, QSizePolicy::Expanding, QSizePolicy::Minimum );
		m_rightSpacer->changeSize(AlloyonHandler::borderSize(), 1, QSizePolicy::Expanding, QSizePolicy::Minimum );
		m_topSpacer->changeSize(1, 1);
		m_bottomSpacer->changeSize(1, AlloyonHandler::borderSize());
		m_leftButtonSpacer->changeSize( 3, 1);
		m_rightButtonSpacer->changeSize( 3, 1);
		m_titleBar->changeSize(1, AlloyonHandler::titleSize(), QSizePolicy::Expanding, QSizePolicy::Fixed);
		m_leftButtonSpacer2->changeSize(1, (AlloyonHandler::titleSize()-AlloyonHandler::buttonSize())/2, QSizePolicy::Minimum, QSizePolicy::Fixed);
		m_rightButtonSpacer2->changeSize(1, (AlloyonHandler::titleSize()-AlloyonHandler::buttonSize())/2, QSizePolicy::Minimum, QSizePolicy::Fixed);

		left = right = AlloyonHandler::borderSize();
		bottom = AlloyonHandler::borderSize();
		top = 1 + AlloyonHandler::titleSize();
	}
	else
	{
		// We've been restored - enlarge the sideways spacers
		m_leftSpacer->changeSize(AlloyonHandler::borderSize(), 1, QSizePolicy::Expanding, QSizePolicy::Minimum );
		m_rightSpacer->changeSize(AlloyonHandler::borderSize(), 1, QSizePolicy::Expanding, QSizePolicy::Minimum );
		m_topSpacer->changeSize(1, 1);
		m_bottomSpacer->changeSize(1, AlloyonHandler::borderSize());
		m_leftButtonSpacer->changeSize( 3, 1);
		m_rightButtonSpacer->changeSize( 3, 1);
		m_titleBar->changeSize(1, AlloyonHandler::titleSize(), QSizePolicy::Expanding, QSizePolicy::Fixed);
		m_leftButtonSpacer2->changeSize(1, (AlloyonHandler::titleSize()-AlloyonHandler::buttonSize())/2, QSizePolicy::Minimum, QSizePolicy::Fixed);
		m_rightButtonSpacer2->changeSize(1, (AlloyonHandler::titleSize()-AlloyonHandler::buttonSize())/2, QSizePolicy::Minimum, QSizePolicy::Fixed);

		left = right = bottom = AlloyonHandler::borderSize();
		top = 1 + AlloyonHandler::titleSize();
	}

	// activate updated layout
	widget()->layout()->activate();
}

void AlloyonClient::reset(unsigned long) // unsigned long changed
{
	// TODO: Implementation
}

void AlloyonClient::resize(const QSize& s)
{
	widget()->resize(s);
}


//////////////// Plugin Stuff

extern "C"
{
	KDecorationFactory *create_factory()
	{
		return new AlloyonHandler();
	}
}

#include "alloyonclient.moc"
