//
// C++ Implementation: alloyonhandler
//
// Description:
//
//
// Author: Simon Ask Ulsnes <simon@ulsnes.dk>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "alloyonhandler.h"
#include "alloyonclient.h"
#include "pixmaps.h"
#include "embeddata.h"

bool AlloyonHandler::m_initialized = false;
bool AlloyonHandler::m_titleShadow = true;
int AlloyonHandler::m_titleShadowSize = 1;
int AlloyonHandler::m_borderSize = 0;
int AlloyonHandler::m_titleSize = 25;
int AlloyonHandler::m_buttonSize = 20;
int AlloyonHandler::m_buttonSpacing = 2;
bool AlloyonHandler::m_redCloseButton = false;

AlloyonHandler::AlloyonHandler()
{
	reset(0);
}


AlloyonHandler::~AlloyonHandler()
{
	Pixmaps::deletePixmaps();
	qCleanupImages_KWinAlloyon();
	m_initialized = false;
}

bool AlloyonHandler::reset(unsigned long) // unsigned long changed
{
	readConfig();

	Pixmaps::deletePixmaps();
	Pixmaps::createPixmaps();
	qInitImages_KWinAlloyon();

	m_initialized = true;
	return true;
}

KDecoration* AlloyonHandler::createDecoration(KDecorationBridge *bridge)
{
	return new AlloyonClient(bridge, this);
}

void AlloyonHandler::readConfig()
{
	KConfig config("kwinalloyonrc");
	config.setGroup("General");

	m_titleShadow = config.readBoolEntry("TitleShadow", true);
	m_redCloseButton = config.readBoolEntry("RedCloseButton", false);

	m_titleShadowSize = config.readNumEntry("TitleShadowSize", 1);
	if (m_titleShadowSize < 1) m_titleShadow = false;
	if (m_titleShadowSize > 3) m_titleShadowSize = 3;

	m_borderSize = config.readNumEntry("BorderSize", 5);
	if (m_borderSize < 0) m_borderSize = 0;
	if (m_borderSize > 5) m_borderSize = 5;

	m_titleSize = config.readNumEntry("TitleSize", 25);
	if (m_titleSize < 16) m_titleSize = 16;
	if (m_titleSize > 48) m_titleSize = 48;

	m_buttonSize = config.readNumEntry("ButtonSize", 20);
	if (m_buttonSize < 12) m_buttonSize = 12;
	if (m_buttonSize > (m_titleSize-4)) m_buttonSize = (m_titleSize-4);

	/*m_buttonSpacing = config.readNumEntry("ButtonSpacing", 2);
	if (m_buttonSpacing < 0) m_buttonSpacing = 0;
	if (m_buttonSize > 5) m_buttonSpacing = 5;*/
}

#include "alloyonhandler.moc"
