//
// C++ Implementation: pixmaps
//
// Description:
//
//
// Author: Simon Ask Ulsnes <simon@ulsnes.dk>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "pixmaps.h"
#include "alloyonhandler.h"

#include <kdecoration.h>
#include <qpainter.h>
#include <kpixmapeffect.h>
#include <kpixmap.h>
#include <kpixmapio.h>
#include <qimage.h>

// Static declaration (for linking) - should conserve a bit memory...
QPixmap *Pixmaps::active_button_common_normal = 0;
QPixmap *Pixmaps::active_button_common_hover = 0;
QPixmap *Pixmaps::active_button_common_sunken = 0;
QPixmap *Pixmaps::active_button_close_normal = 0;
QPixmap *Pixmaps::active_button_close_hover = 0;
QPixmap *Pixmaps::active_button_close_sunken = 0;

QPixmap *Pixmaps::inactive_button_common_normal = 0;
QPixmap *Pixmaps::inactive_button_common_hover = 0;
QPixmap *Pixmaps::inactive_button_common_sunken = 0;
QPixmap *Pixmaps::inactive_button_close_normal = 0;
QPixmap *Pixmaps::inactive_button_close_hover = 0;
QPixmap *Pixmaps::inactive_button_close_sunken = 0;

QPixmap *Pixmaps::active_titlebar = 0;
QPixmap *Pixmaps::active_titlebar_left = 0;
QPixmap *Pixmaps::active_titlebar_right = 0;
QPixmap *Pixmaps::inactive_titlebar = 0;
QPixmap *Pixmaps::inactive_titlebar_left = 0;
QPixmap *Pixmaps::inactive_titlebar_right = 0;

QPixmap *Pixmaps::active_resizer_detail_5px = 0;   // 5 px border
QPixmap *Pixmaps::inactive_resizer_detail_5px = 0; // 5 px border
QPixmap *Pixmaps::active_resizer_detail_4px = 0;   // 4 px border
QPixmap *Pixmaps::inactive_resizer_detail_4px = 0; // 4 px border

bool Pixmaps::pixmapsCreated = false;

void Pixmaps::createPixmaps()
{
	if (pixmapsCreated)
		return;

	QColor aColor = KDecoration::options()->color(KDecoration::ColorTitleBar, true);
	QColor iColor = KDecoration::options()->color(KDecoration::ColorTitleBar, false);
	//QColor aCloseColor = QColor(220, 92, 92);
	QColor aCloseColor = aColor;

	if (AlloyonHandler::redCloseButton())
	{
		int ac_h, ac_s, ac_v;
		aColor.getHsv(&ac_h, &ac_s, &ac_v);
		aCloseColor.setHsv(0, ac_s, ac_v);
	}

	QColor iCloseColor = iColor;

	int buttonSize = AlloyonHandler::buttonSize();
	int titleSize = AlloyonHandler::titleSize();

	// This object's going to do all the painting.
	QPainter painter;

	// This object will host the graphics while we're painting.
	KPixmap tempPixmap;

	active_titlebar = new QPixmap(1, titleSize-1);
	painter.begin(active_titlebar);
	tempPixmap.resize(1, titleSize-1);
	KPixmapEffect::gradient(tempPixmap, hsvRelative(aColor, 0, 20, -50), aColor, KPixmapEffect::VerticalGradient);
	painter.drawPixmap(0, 0, tempPixmap);
	painter.end();

	inactive_titlebar = new QPixmap(1, titleSize-1);
	painter.begin(inactive_titlebar);
	tempPixmap.resize(1, titleSize-1);
	KPixmapEffect::gradient(tempPixmap, hsvRelative(iColor, 0, 20, -50), iColor, KPixmapEffect::VerticalGradient);
	painter.drawPixmap(0, 0, tempPixmap);
	painter.end();

	active_titlebar_left = new QPixmap(1, titleSize-1);
	painter.begin(active_titlebar_left);
	tempPixmap.resize(1, titleSize-1);
	KPixmapEffect::gradient(tempPixmap, hsvRelative(aColor, 0, -50, +20), hsvRelative(aColor, 0, -80, +40), KPixmapEffect::VerticalGradient);
	painter.drawPixmap(0, 0, tempPixmap);
	painter.end();

	inactive_titlebar_left = new QPixmap(1, titleSize-1);
	painter.begin(inactive_titlebar_left);
	tempPixmap.resize(1, titleSize-1);
	KPixmapEffect::gradient(tempPixmap, hsvRelative(iColor, 0, -50, +20), hsvRelative(iColor, 0, -80, +40), KPixmapEffect::VerticalGradient);
	painter.drawPixmap(0, 0, tempPixmap);
	painter.end();

	active_titlebar_right = new QPixmap(1, titleSize-1);
	painter.begin(active_titlebar_right);
	tempPixmap.resize(1, titleSize-1);
	KPixmapEffect::gradient(tempPixmap, hsvRelative(aColor, 0, 0, -50), hsvRelative(aColor, 0, -80, -50), KPixmapEffect::VerticalGradient);
	painter.drawPixmap(0, 0, tempPixmap);
	painter.end();

	inactive_titlebar_right = new QPixmap(1, titleSize-1);
	painter.begin(inactive_titlebar_right);
	tempPixmap.resize(1, titleSize-1);
	KPixmapEffect::gradient(tempPixmap, hsvRelative(iColor, 0, 0, -50), hsvRelative(iColor, 0, -80, -50), KPixmapEffect::VerticalGradient);
	painter.drawPixmap(0, 0, tempPixmap);
	painter.end();

	// Now we're going to paint the resize handle pixmaps.

	active_resizer_detail_5px = new QPixmap(3, 3);
	painter.begin(active_resizer_detail_5px);
	painter.setPen(hsvRelative(aColor, 0, 0, -50) );
	painter.drawLine(0, 0, 1, 0);
	painter.drawPoint(0, 1);
	painter.setPen(hsvRelative(aColor, 0, -50, +20) );
	painter.drawPoint(2, 1);
	painter.drawLine(1, 2, 2, 2);
	painter.setPen(aColor);
	painter.drawPoint(2, 0);
	painter.drawPoint(1, 1);
	painter.drawPoint(0, 2);
	painter.end();

	inactive_resizer_detail_5px = new QPixmap(3, 3);
	painter.begin(inactive_resizer_detail_5px);
	painter.setPen(hsvRelative(iColor, 0, 0, -50) );
	painter.drawLine(0, 0, 1, 0);
	painter.drawPoint(0, 1);
	painter.setPen(hsvRelative(iColor, 0, -50, +20) );
	painter.drawPoint(2, 1);
	painter.drawLine(1, 2, 2, 2);
	painter.setPen(iColor);
	painter.drawPoint(2, 0);
	painter.drawPoint(1, 1);
	painter.drawPoint(0, 2);
	painter.end();

	active_resizer_detail_4px = new QPixmap(2, 2);
	painter.begin(active_resizer_detail_4px);
	painter.setPen(hsvRelative(aColor, 0, 0, -50) );
	painter.drawLine(0, 0, 1, 0);
	painter.drawPoint(0, 1);
	painter.setPen(hsvRelative(aColor, 0, -50, +20) );
	painter.drawPoint(1, 1);
	painter.end();

	inactive_resizer_detail_4px = new QPixmap(2, 2);
	painter.begin(inactive_resizer_detail_4px);
	painter.setPen(hsvRelative(iColor, 0, 0, -50) );
	painter.drawLine(0, 0, 1, 0);
	painter.drawPoint(0, 1);
	painter.setPen(hsvRelative(iColor, 0, -50, +20) );
	painter.drawPoint(1, 1);
	painter.end();

	// Now we'll create the button pixmaps.
	active_button_common_normal = new QPixmap(buttonSize, buttonSize);
	painter.begin(active_button_common_normal);
	drawButton(&painter, aColor, aColor, 1);
	painter.end();
	inactive_button_common_normal = new QPixmap(buttonSize, buttonSize);
	painter.begin(inactive_button_common_normal);
	drawButton(&painter, iColor, iColor, 1, false);
	painter.end();

	active_button_common_hover = new QPixmap(buttonSize, buttonSize);
	painter.begin(active_button_common_hover);
	drawButton(&painter, aColor, aColor, 2);
	painter.end();
	inactive_button_common_hover = new QPixmap(buttonSize, buttonSize);
	painter.begin(inactive_button_common_hover);
	drawButton(&painter, iColor, iColor, 2, false);
	painter.end();

	active_button_common_sunken = new QPixmap(buttonSize, buttonSize);
	painter.begin(active_button_common_sunken);
	drawButton(&painter, aColor, aColor, 3);
	painter.end();
	inactive_button_common_sunken = new QPixmap(buttonSize, buttonSize);
	painter.begin(inactive_button_common_sunken);
	drawButton(&painter, iColor, iColor, 3, false);
	painter.end();

	// ...and the red ones for the close button...
	active_button_close_normal = new QPixmap(buttonSize, buttonSize);
	painter.begin(active_button_close_normal);
	drawButton(&painter, aColor, aColor, 1);
	painter.end();
	inactive_button_close_normal = new QPixmap(buttonSize, buttonSize);
	painter.begin(inactive_button_close_normal);
	drawButton(&painter, iCloseColor, iColor, 1, false);
	painter.end();

	active_button_close_hover = new QPixmap(buttonSize, buttonSize);
	painter.begin(active_button_close_hover);
	drawButton(&painter, aColor, aCloseColor, 2);
	painter.end();
	inactive_button_close_hover = new QPixmap(buttonSize, buttonSize);
	painter.begin(inactive_button_close_hover);
	drawButton(&painter, iCloseColor, iColor, 2, false);
	painter.end();

	active_button_close_sunken = new QPixmap(buttonSize, buttonSize);
	painter.begin(active_button_close_sunken);
	drawButton(&painter, aColor, aCloseColor, 3);
	painter.end();
	inactive_button_close_sunken = new QPixmap(buttonSize, buttonSize);
	painter.begin(inactive_button_close_sunken);
	drawButton(&painter, iCloseColor, iColor, 3, false);
	painter.end();
}

void Pixmaps::deletePixmaps()
{
	delete active_button_common_normal;
	active_button_common_normal = 0;
	delete active_button_common_hover;
	active_button_common_hover = 0;
	delete active_button_common_sunken;
	active_button_common_sunken = 0;

	delete inactive_button_common_normal;
	inactive_button_common_normal = 0;
	delete inactive_button_common_hover;
	inactive_button_common_hover = 0;
	delete inactive_button_common_sunken;
	inactive_button_common_sunken = 0;

	delete active_button_close_normal;
	active_button_close_normal = 0;
	delete active_button_close_hover;
	active_button_close_hover = 0;
	delete active_button_close_sunken;
	active_button_close_sunken = 0;

	delete inactive_button_close_normal;
	inactive_button_close_normal = 0;
	delete inactive_button_close_hover;
	inactive_button_close_hover = 0;
	delete inactive_button_close_sunken;
	inactive_button_close_sunken = 0;

	delete active_titlebar;
	active_titlebar = 0;
	delete active_titlebar_left;
	active_titlebar_left = 0;
	delete active_titlebar_right;
	active_titlebar_right = 0;

	delete inactive_titlebar;
	inactive_titlebar = 0;
	delete inactive_titlebar_left;
	inactive_titlebar_left = 0;
	delete inactive_titlebar_right;
	inactive_titlebar_right = 0;

	delete active_resizer_detail_5px;
	active_resizer_detail_5px = 0;
	delete inactive_resizer_detail_5px;
	inactive_resizer_detail_5px = 0;
	delete active_resizer_detail_4px;
	active_resizer_detail_4px = 0;
	delete inactive_resizer_detail_4px;
	inactive_resizer_detail_4px = 0;

	pixmapsCreated = false;
}

void Pixmaps::drawButton(QPainter* painter, QColor& color, QColor& blendColor, int state, bool active)
{
	if (state < 1 || state > 3)
		return;

	int buttonSize = AlloyonHandler::buttonSize();
	int titleSize = AlloyonHandler::titleSize();

	// Get the titlebar colors right behind the buttons (for the gradient later)
	QImage *titlebar_image = new QImage();
	*titlebar_image = active ? active_titlebar->convertToImage() : inactive_titlebar->convertToImage();
	QColor title_color1 = titlebar_image->pixel(0, (titleSize/2)-(buttonSize/2));
	QColor title_color2 = titlebar_image->pixel(0, (titleSize/2)+(buttonSize/2));

	// Loop integer - memory conservation...
	int i;

	// The temporary canvas.
	KPixmap tempPixmap;

	switch (state)
	{
		case 1:	// normal
			tempPixmap.resize(buttonSize, buttonSize);
			//KPixmapEffect::gradient(tempPixmap, hsvRelative(color, 0, 20, -50), color, KPixmapEffect::VerticalGradient);
			KPixmapEffect::gradient(tempPixmap, title_color1, title_color2, KPixmapEffect::VerticalGradient);
			painter->drawPixmap(0, 0, tempPixmap);
			break;
		case 3: // sunken
		case 2:	// hover
			tempPixmap.resize(buttonSize, buttonSize);
			KPixmapEffect::gradient(tempPixmap, hsvRelative(color, 0, -40, 10), hsvRelative(blendColor, 0, -40, 10), KPixmapEffect::VerticalGradient);
			painter->drawPixmap(0, 0, tempPixmap);
			// draw the button outline
			painter->setPen(hsvRelative(title_color2, 0, -80, 30));
			painter->drawLine(0, 1, 0, buttonSize-2); // left
			painter->drawLine(1, 0, buttonSize-2, 0); // top
			painter->drawLine(buttonSize-1, 1, buttonSize-1, buttonSize-2); // right
			painter->drawLine(1, buttonSize-1, buttonSize-2, buttonSize-1); // bottom
			// draw the correct colors in the corners
			painter->setPen(title_color1);
			painter->drawPoint(0, 0);
			painter->drawPoint(buttonSize-1, 0);
			painter->setPen(title_color2);
			painter->drawPoint(0, buttonSize-1);
			painter->drawPoint(buttonSize-1, buttonSize-1);
			break;
		default:
			break;
	}
	delete titlebar_image;
}

QColor Pixmaps::hsvRelative(QColor& baseColor, int relH, int relS, int relV)
{
	int h, s, v;
	baseColor.hsv(&h, &s, &v);

	h = h + relH;
	s = s + relS;
	v = v + relV;

	if (h < 0)
		h = 0;
	else if (h > 359)
		h = 359;

	if (s < 0)
		s = 0;
	else if (s > 255)
		s = 255;

	if (v < 0)
		v = 0;
	else if (v > 255)
		v = 255;

	QColor c;
	c.setHsv( h, s, v );
	return c;
}
